@@ -2,24 +2,26 @@
 from __future__ import annotations
 
 # Somewhat hacky type checking to avoid circular imports:
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Iterable
 if TYPE_CHECKING:
     from monaco.mc_sim import Sim
 
 import numpy as np
-from monaco.helper_functions import vprint
 from scipy.optimize import minimize
 from warnings import warn
+from monaco.helper_functions import vprint, get_cases
 
 # numba is recommended for speed, as this will be very slow otherwise
 try:
     from numba import jit
+    HAS_NUMBA = True
 except ImportError:
-    warn("Consider installing numba for better performance", UserWarning)
+    HAS_NUMBA = False
+
     # create a dummy decorator
     def jit(f=None, *args, **kwargs):
         def decorator(func):
-            return func 
+            return func
 
         if callable(f):
             return f
@@ -29,6 +31,7 @@ except ImportError:
 
 def calc_sensitivities(sim        : 'Sim',
                        outvarname : str,
+                       cases      : None | int | Iterable[int] = None,
                        Hj         : float = 1.0,
                        tol        : float = 1e-6,
                        verbose    : bool  = False,
@@ -51,6 +54,9 @@ def calc_sensitivities(sim        : 'Sim',
     outvarname : str
         The name of the output variable to calculate sensitivities for. Note
         that the output variable must be scalar.
+    cases : None | int | Iterable[int], default None
+        The cases to use for the sensitivity analysis. If None, all cases will
+        be used.
     Hj : float, default: 1.0
         The fraction of the total parameter space to integrate over. Note that
         the linear correlation function only has one hyperparameter, so as the
@@ -83,9 +89,17 @@ def calc_sensitivities(sim        : 'Sim',
            comprehensive, robust, and efficient global sensitivity analysis:
            1. Theory." Water Resources Research 52.1 (2016): 423-439.
     """
-    phi_opt = calc_phi_opt(sim, outvarname, tol, verbose)
+    if not HAS_NUMBA:
+        warn("Consider installing numba for better performance", UserWarning)
 
-    variance = np.var(np.array([sim.outvars[outvarname].nums]))
+    cases = get_cases(ncases=sim.ncases, cases=cases)
+    ncases = len(cases)
+    if ncases != sim.ncases:
+        warn(f'Only using {ncases} of {sim.ncases} cases for D-VARS sensitivity analysis')
+
+    phi_opt = calc_phi_opt(sim, outvarname, cases, tol, verbose)
+
+    variance = np.var(np.array(sim.outvars[outvarname].nums)[cases])
     sensitivities = np.zeros(sim.ninvars)
     for j in range(sim.ninvars):
         sensitivities[j] = calc_Gammaj(Hj, phi_opt[j], variance)
@@ -96,6 +110,7 @@ def calc_sensitivities(sim        : 'Sim',
 
 def calc_phi_opt(sim        : 'Sim',
                  outvarname : str,
+                 cases      : list[int],
                  tol        : float = 1e-6,
                  verbose    : bool = False
                  ) -> np.ndarray:
@@ -112,6 +127,8 @@ def calc_phi_opt(sim        : 'Sim',
     outvarname : str
         The name of the output variable to calculate sensitivities for. Note
         that the output variable must be scalar.
+    cases : list[int]
+        The cases to use for the sensitivity analysis.
     tol : float, default 1e-6
         The convergence tolerance for scipy's minimize function acting on the
         negative log likelihood function.
@@ -135,7 +152,7 @@ def calc_phi_opt(sim        : 'Sim',
 
     vprint(sim.verbose, 'Calculating optimal hyperparameters Φ for ' +
                        f"'{outvarname}' covariances...")
-    X, Y = full_states(sim, outvarname)
+    X, Y = full_states(sim, outvarname, cases)
     res = minimize(L_runner, phi0s, args=(X, Y, verbose), bounds=bounds,
                    tol=tol, method='L-BFGS-B')
     phi_opt = res.x
@@ -146,6 +163,7 @@ def calc_phi_opt(sim        : 'Sim',
 
 def full_states(sim : 'Sim',
                 outvarname : str,
+                cases      : list[int]
                 ) -> tuple[np.ndarray, np.ndarray]:
     """
     Get the full input and output states in combined matrices.
@@ -157,6 +175,8 @@ def full_states(sim : 'Sim',
     outvarname : str
         The name of the output variable to calculate sensitivities for. Note
         that the output variable must be scalar.
+    cases : list[int]
+        The cases to use for the sensitivity analysis.
 
     Returns
     -------
@@ -165,11 +185,12 @@ def full_states(sim : 'Sim',
     Y : numpy.ndarray
         The output variable's nums.
     """
-    X = np.zeros((sim.ncases, sim.ninvars))
-    Y = np.zeros((sim.ncases, 1))
+    ncases = len(cases)
+    X = np.zeros((ncases, sim.ninvars))
+    Y = np.zeros((ncases, 1))
     for i, varname in enumerate(sim.invars):
-        X[:, i] = sim.invars[varname].pcts
-    Y[:, 0] = sim.outvars[outvarname].nums
+        X[:, i] = np.array(sim.invars[varname].pcts)[cases]
+    Y[:, 0] = np.array(sim.outvars[outvarname].nums)[cases]
     return X, Y
 
 
@@ -299,7 +320,7 @@ def calc_R(phi : np.ndarray,
         The correlation matrix.
     """
     m = X.shape[0]
-    R = np.ones((m, m))
+    R = np.ones((m, m), dtype=np.float64)
     for u in range(1, m):
         # do lower triangle only and duplicate across diag
         # diag will be all 1s
