@@ -1,17 +1,19 @@
 # mc_case.py
 from __future__ import annotations
 
-from monaco.mc_var import OutVar, InVar
-from monaco.mc_val import OutVal, InVal
+import numpy as np
 from datetime import datetime, timedelta
 from pathlib import Path
 from typing import Any
-import numpy as np
+from monaco.mc_var import OutVar, InVar
+from monaco.mc_val import OutVal, InVal
 
 class Case():
     """
     Object to hold all the data for a single Monte Carlo case.
 
+    `InVal`s and `OutVal`s can be accessed by name, eg `case['Var1']`.
+
     Parameters
     ----------
     ncase : int
@@ -48,10 +50,12 @@ class Case():
         Whether this case has run the run function.
     haspostprocessed : bool
         Whether this case has been postprocessed.
+    constvals : dict[str, Any]
+        The constant values for this case.
     invals : dict[str, monaco.mc_val.InVal]
-        The input values for this partitcular case.
+        The input values for this particular case.
     outvals : dict[str, monaco.mc_val.OutVal]
-        The output values for this partitcular case.
+        The output values for this particular case.
     siminput : tuple[Any]
         The preprocessed inputs provided to the run function for this case.
     simrawoutput : tuple[Any]
@@ -61,7 +65,7 @@ class Case():
                  ncase            : int,
                  ismedian         : bool,
                  invars           : dict[str, InVar],
-                 constvals        : dict[str, Any] = None,
+                 constvals        : dict[str, Any] | None = None,
                  keepsiminput     : bool = True,
                  keepsimrawoutput : bool = True,
                  seed             : int = np.random.get_state(legacy=False)['state']['key'][0],
@@ -70,6 +74,7 @@ class Case():
         self.ncase = ncase
         self.ismedian = ismedian
         self.invars = invars
+        self.vars : dict[str, InVar | OutVar] = dict(invars)
         if constvals is None:
             constvals = dict()
         self.constvals = constvals
@@ -78,21 +83,22 @@ class Case():
         self.keepsimrawoutput = keepsimrawoutput
         self.seed = seed
 
-        self.starttime : datetime = None
-        self.endtime   : datetime = None
-        self.runtime   : timedelta = None
+        self.starttime : datetime | None = None
+        self.endtime   : datetime | None = None
+        self.runtime   : timedelta | None = None
 
-        self.filepath : Path = None
-        self.runsimid : int = None
+        self.filepath : Path | None = None
+        self.runsimid : int | None = None
         self.haspreprocessed  : bool = False
         self.hasrun           : bool = False
         self.haspostprocessed : bool = False
 
         self.invals  : dict[str, InVal]  = self.getInVals()
         self.outvals : dict[str, OutVal] = dict()
+        self.vals    : dict[str, InVal | OutVal] = dict(self.invals)
 
-        self.siminput     : tuple[Any] = None
-        self.simrawoutput : tuple[Any] = None
+        self.siminput     : tuple[Any] | None = None
+        self.simrawoutput : tuple[Any] | None = None
 
 
     def __repr__(self):
@@ -100,6 +106,24 @@ class Case():
                 f"\n  outvals={self.outvals})")
 
 
+    def __getitem__(self,
+                    valname : str,
+                    ) -> InVal | OutVal:
+        """Get a InVal or OutVal from the case.
+
+        Parameters
+        ----------
+        valname : str
+            The name of the value to get.
+
+        Returns
+        -------
+        val : InVal | OutVal
+            The value requested.
+        """
+        return self.vals[valname]
+
+
     def getInVals(self) -> dict[str, InVal]:
         """
         From all the InVar's, extract the vals for this case.
@@ -116,6 +140,20 @@ class Case():
         return vals
 
 
+    def addOutVar(self,
+                  outvar : OutVar,
+                  ) -> None:
+        """Add an OutVar to the case.
+
+        Parameters
+        ----------
+        outvar : OutVar
+            The OutVar to add.
+        """
+        self.outvars[outvar.name] = outvar
+        self.vars[outvar.name] = outvar
+
+
     def getOutVals(self) -> dict[str, OutVal]:
         """
         From all the OutVar's, extract the vals for this case.
@@ -136,7 +174,7 @@ class Case():
                   name   : str,
                   val    : Any,
                   split  : bool = True,
-                  valmap : dict[Any, float] = None
+                  valmap : dict[Any, float] | None = None
                   ) -> None:
         """
         Generate an OutVal and add it to the dict of outvals.
@@ -155,7 +193,10 @@ class Case():
         if name in self.outvals.keys():
             raise ValueError(f"'{name}' is already an OutVal")
 
-        self.outvals[name] = OutVal(name=name, ncase=self.ncase, val=val,
-                                    valmap=valmap, ismedian=self.ismedian)
+        outval = OutVal(name=name, ncase=self.ncase, val=val,
+                        valmap=valmap, ismedian=self.ismedian)
+        self.outvals[name] = outval
+        self.vals[name] = outval
         if split:
-            self.outvals.update(self.outvals[name].split())
+            self.outvals.update(outval.split())
+            self.vals.update(outval.split())
