@@ -1,14 +1,15 @@
 # mc_multi_plot.py
 from __future__ import annotations
 
+import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.figure import Figure
 from matplotlib.axes import Axes
+from typing import Optional, Iterable
 from monaco.mc_plot import plot_hist, plot_2d_scatter
 from monaco.mc_var import InVar, OutVar
 from monaco.mc_enums import PlotOrientation, InVarSpace
 from monaco.helper_functions import empty_list, get_list
-from typing import Optional, Iterable
 
 
 def multi_plot(vars        : list[InVar | OutVar],
@@ -18,7 +19,7 @@ def multi_plot(vars        : list[InVar | OutVar],
                cov_plot    : bool   = False,
                cov_p       : None | float | Iterable[float] = None,
                invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
-               fig         : Figure = None,
+               fig         : Figure | None = None,
                title       : str    = '',
                plotkwargs  : dict   = dict(),
                ) -> tuple[Figure, tuple[Axes, ...]]:
@@ -104,7 +105,7 @@ def multi_plot_2d_scatter_hist(varx     : InVar | OutVar,
                                cov_p      : None | float | Iterable[float] = None,
                                cumulative : bool   = False,
                                invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
-                               fig        : Figure = None,
+                               fig        : Figure | None = None,
                                title      : str    = '',
                                plotkwargs : dict   = dict(),
                                ) -> tuple[Figure, tuple[Axes, ...]]:
@@ -189,7 +190,7 @@ def multi_plot_grid_tri(vars     : list[InVar | OutVar],
                         cov_p      : None | float | Iterable[float] = None,
                         cumulative : bool   = False,
                         invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
-                        fig        : Figure = None,
+                        fig        : Figure | None = None,
                         title      : str    = '',
                         plotkwargs : dict   = dict(),
                         ) -> tuple[Figure, tuple[Axes, ...]]:
@@ -234,7 +235,7 @@ def multi_plot_grid_tri(vars     : list[InVar | OutVar],
     fig = handle_fig(fig)
 
     nvars = len(vars)
-    axs = fig.subplots(nvars, nvars, sharex='col')
+    axs = np.atleast_2d(fig.subplots(nvars, nvars, sharex='col'))
     for i in range(nvars):
         row_scatter_axs = [ax for k, ax in enumerate(axs[i]) if k < i]
         for j in range(nvars):
@@ -247,7 +248,8 @@ def multi_plot_grid_tri(vars     : list[InVar | OutVar],
             elif j > i:
                 fig.delaxes(ax)
             else:
-                ax.get_shared_y_axes().join(*row_scatter_axs)  # Don't link the histogram y axis
+                # Don't link the histogram y axis
+                ax.get_shared_y_axes()._grouper.join(*row_scatter_axs)
                 plot_2d_scatter(vars[j], vars[i],
                                 cases=cases, highlight_cases=highlight_cases,
                                 rug_plot=rug_plot, cov_plot=cov_plot, cov_p=cov_p,
@@ -278,7 +280,7 @@ def multi_plot_grid_rect(varsx     : InVar | OutVar | list[InVar | OutVar],
                          cov_p      : None | float | Iterable[float] = None,
                          cumulative : bool   = False,
                          invar_space : InVarSpace | Iterable[InVarSpace] = InVarSpace.NUMS,
-                         fig        : Figure = None,
+                         fig        : Figure | None = None,
                          title      : str    = '',
                          plotkwargs : dict   = dict(),
                          ) -> tuple[Figure, tuple[Axes, ...]]:
@@ -328,7 +330,7 @@ def multi_plot_grid_rect(varsx     : InVar | OutVar | list[InVar | OutVar],
     nvarsx = len(varsx)
     nvarsy = len(varsy)
 
-    axs = fig.subplots(nvarsy+1, nvarsx+1, sharex='col')
+    axs = np.atleast_2d(fig.subplots(nvarsy+1, nvarsx+1, sharex='col'))
     for i in range(nvarsy+1):
         row_scatter_axs = [ax for k, ax in enumerate(axs[i]) if k < nvarsy+1]
         for j in range(nvarsx+1):
@@ -347,7 +349,8 @@ def multi_plot_grid_rect(varsx     : InVar | OutVar | list[InVar | OutVar],
                           ax=ax, title='', plotkwargs=plotkwargs)
                 ax.set_xticklabels([])
             else:
-                ax.get_shared_y_axes().join(*row_scatter_axs)  # Don't link the histogram y axis
+                # Don't link the histogram y axis
+                ax.get_shared_y_axes()._grouper.join(*row_scatter_axs)
                 plot_2d_scatter(varsx[j-1], varsy[i],
                                 cases=cases, highlight_cases=highlight_cases,
                                 rug_plot=rug_plot, cov_plot=cov_plot, cov_p=cov_p,
