@@ -17,7 +17,7 @@ from matplotlib.figure import Figure
 from matplotlib.axes import Axes
 from matplotlib.patches import Ellipse
 from mpl_toolkits.mplot3d import Axes3D  # noqa: F401
-from monaco.helper_functions import get_list, slice_by_index, length, empty_list
+from monaco.helper_functions import get_list, slice_by_index, length, empty_list, get_cases
 from monaco.gaussian_statistics import conf_ellipsoid_sig2pct
 from monaco.integration_statistics import integration_error
 from monaco.mc_enums import SampleMethod, PlotOrientation, InVarSpace, Sensitivities
@@ -249,6 +249,12 @@ def plot_hist(var         : InVar | OutVar,
     counts, bins = np.histogram(nums, bins=bins)
 
     if isinstance(var, monaco.mc_var.InVar):
+        # Loaded from file
+        if var.dist is None:
+            plt.hist(bins[:-1], bins=bins, weights=counts/sum(counts), density=True,
+                     cumulative=cumulative, orientation=orientation, histtype='bar',
+                     facecolor='k', alpha=0.5, **plotkwargs)
+
         # Continuous distribution
         if isinstance(var.dist, rv_continuous):
             plt.hist(bins[:-1], bins=bins, weights=counts/sum(counts), density=True,
@@ -860,6 +866,7 @@ def plot_integration_convergence(outvar       : OutVar,
                                  dimension    : int,
                                  volume       : float,
                                  refval       : float          = None,
+                                 cases        : None | int | Iterable[int] = None,
                                  conf         : float          = 0.95,
                                  samplemethod : SampleMethod   = SampleMethod.RANDOM,
                                  ax           : Optional[Axes] = None,
@@ -879,7 +886,10 @@ def plot_integration_convergence(outvar       : OutVar,
     volume : float
         The total integration volume.
     refval : float, default: None
-        If known a-priori, the reference value for the integration.
+        If known a priori, the reference value for the integration.
+    cases : None | int | Iterable[int], default: None
+        The cases to use for the integration convergence plot. If None, all
+        cases will be used.
     conf : float, default: 0.95
         The confidence level for the error estimate. 0.95 corresponds to 95%.
     samplemethod : monaco.mc_enums.SampleMethod
@@ -897,11 +907,15 @@ def plot_integration_convergence(outvar       : OutVar,
     """
     fig, ax = manage_axis(ax, is3d=False)
 
+    cases = get_cases(ncases=outvar.ncases, cases=cases)
+    ncases = len(cases)
+
     if refval is not None:
         ax.axhline(refval, color='k')
 
-    cummean = volume*np.cumsum(outvar.nums)/np.arange(1, outvar.ncases+1)
-    err = integration_error(nums=np.array(outvar.nums), dimension=dimension, volume=volume,
+    nums = np.array(outvar.nums)[cases]
+    cummean = volume*np.cumsum(nums)/np.arange(1, ncases+1)
+    err = integration_error(nums=nums, dimension=dimension, volume=volume,
                             conf=conf, samplemethod=samplemethod, runningerror=True)
     ax.plot(cummean, 'C1')
     ax.plot(cummean + err, 'C0')
@@ -919,6 +933,7 @@ def plot_integration_error(outvar       : OutVar,
                            dimension    : int,
                            volume       : float,
                            refval       : float,
+                           cases        : None | int | Iterable[int] = None,
                            conf         : float          = 0.95,
                            samplemethod : SampleMethod   = SampleMethod.RANDOM,
                            ax           : Optional[Axes] = None,
@@ -940,6 +955,9 @@ def plot_integration_error(outvar       : OutVar,
         The total integration volume.
     refval : float
         The reference value for the integration.
+    cases : None | int | Iterable[int], default: None
+        The cases to use for the integration error plot. If None, all cases
+        will be used.
     conf : float, default: 0.95
         The confidence level for the error estimate. 0.95 corresponds to 95%.
     samplemethod : monaco.mc_enums.SampleMethod
@@ -957,8 +975,12 @@ def plot_integration_error(outvar       : OutVar,
     """
     fig, ax = manage_axis(ax, is3d=False)
 
-    cummean = volume*np.cumsum(outvar.nums)/np.arange(1, outvar.ncases+1)
-    err = integration_error(nums=np.array(outvar.nums), dimension=dimension, volume=volume,
+    cases = get_cases(ncases=outvar.ncases, cases=cases)
+    ncases = len(cases)
+
+    nums = np.array(outvar.nums)[cases]
+    cummean = volume*np.cumsum(nums)/np.arange(1, ncases+1)
+    err = integration_error(nums=nums, dimension=dimension, volume=volume,
                             conf=conf, samplemethod=samplemethod, runningerror=True)
     ax.loglog(err, 'C0')
     ax.plot(np.abs(cummean - refval), 'C1')
@@ -1061,9 +1083,9 @@ def manage_axis(ax   : Optional[Axes],
 
 
 def apply_category_labels(ax   : Axes,
-                          varx : InVar | OutVar = None,
-                          vary : InVar | OutVar = None,
-                          varz : InVar | OutVar = None,
+                          varx : InVar | OutVar | None = None,
+                          vary : InVar | OutVar | None = None,
+                          varz : InVar | OutVar | None = None,
                           ) -> None:
     """
     For nonnumeric Monte Carlo variables, use the `nummap` to label the axes.
@@ -1159,10 +1181,11 @@ def plot_rug_marks(ax          : Axes,
 
 
 
-def plot_2d_cov_ellipse(ax   : Axes,
-                        varx : InVar | OutVar,
-                        vary : InVar | OutVar,
-                        p    : float,
+def plot_2d_cov_ellipse(ax    : Axes,
+                        varx  : InVar | OutVar,
+                        vary  : InVar | OutVar,
+                        p     : float,
+                        cases : None | int | Iterable[int] = None,
                         ) -> None:
     """
     Add a covariance ellipse to a 2D scatter plot.
@@ -1177,13 +1200,18 @@ def plot_2d_cov_ellipse(ax   : Axes,
         The y variable.
     p : float
         Coviariance percentile, assuming a gaussian distribution.
+    cases : None | int | Iterable[int], default: None
+        The cases to use for the covariance ellipse. If None, all cases will
+        be used.
     """
     if ax is None:
         return
 
+    cases = get_cases(ncases=varx.ncases, cases=cases)
+
     # See https://www.visiondummy.com/2014/04/draw-error-ellipse-representing-covariance-matrix/
-    allnums = [np.array(varx.nums), np.array(vary.nums)]
-    center = [np.mean(varx.nums), np.mean(vary.nums)]
+    allnums = [np.array(varx.nums)[cases], np.array(vary.nums)[cases]]
+    center = [np.mean(allnums[0]), np.mean(allnums[1])]
 
     covs = np.cov(np.array(allnums))
     eigvals, eigvecs = np.linalg.eigh(covs)  # Use eigh over eig since covs is guaranteed symmetric
@@ -1204,33 +1232,6 @@ def plot_2d_cov_ellipse(ax   : Axes,
     plt.axline(xy1=(center[0], center[1]), slope=eigvecs[1][1]/eigvecs[1][0], color='k')
 
 
-
-def get_cases(ncases : int,
-              cases  : None | int | Iterable[int],
-              ) -> list[int]:
-    """
-    Parse the `cases` input for plotting functions. If None, return a list of
-    all the cases. Otherwise, return a list of all the specified cases.
-
-    Parameters
-    ----------
-    ncases : int
-        The total number of cases.
-    cases : None | int | Iterable[int]
-        The cases to downselect to.
-
-    Returns
-    -------
-    cases_list : list[int]
-        The cases.
-    """
-    if cases is None:
-        cases = list(range(ncases))
-    cases_list = get_list(cases)
-    return cases_list
-
-
-
 def manage_invar_space(invar_space : InVarSpace | Iterable[InVarSpace],
                        nvars       : int,
                        ) -> list[InVarSpace]:
