@@ -1,18 +1,18 @@
 # mc_sampling.py
 from __future__ import annotations
 
+import warnings
+import sys
 import scipy.stats
 import numpy as np
 from functools import lru_cache
-import warnings
-import sys
 from monaco.mc_enums import SampleMethod
 
 
 def sampling(ndraws     : int,
              method     : SampleMethod = SampleMethod.SOBOL_RANDOM,
-             ninvar     : int          = None,
-             ninvar_max : int          = None,
+             ninvar     : int | None   = None,
+             ninvar_max : int | None   = None,
              seed       : int          = np.random.get_state(legacy=False)['state']['key'][0],
              ) -> np.ndarray:
     """
@@ -113,7 +113,8 @@ def cached_pcts(ndraws     : int,
             # Suppress the power of 2 warning for sobol / halton sequences
             warnings.simplefilter("ignore", category=UserWarning)
             points = sampler.random(n=ndraws)
+    else:
+        points = sampler.random(n=ndraws)
 
     all_pcts = np.array(points)
-
     return all_pcts
