@@ -3,12 +3,10 @@ from __future__ import annotations
 
 import os
 import numpy as np
-import dask
 import csv
 import json
 import cloudpickle
 import pathlib
-from dask.distributed import Client
 from datetime import datetime, timedelta
 from matplotlib.figure import Figure
 from matplotlib.axes import Axes
@@ -21,15 +19,30 @@ from monaco.mc_enums import SimFunctions, SampleMethod
 from monaco.helper_functions import (get_list, vprint, vwarn, empty_list,
                                      hash_str_repeatable)
 from monaco.case_runners import preprocess_case, run_case, postprocess_case
-from monaco.tqdm_dask_distributed import tqdm_dask
 from monaco.dvars_sensitivity import calc_sensitivities
 from monaco.mc_multi_plot import multi_plot_grid_rect
 
+try:
+    import dask
+    from dask.distributed import Client, progress
+    HAS_DASK = True
+except ImportError:
+    HAS_DASK = False
+
+try:
+    import pandas as pd
+    HAS_PANDAS = True
+except ImportError:
+    HAS_PANDAS = False
+
 
 class Sim:
     """
     The main Monte Carlo Simulation object.
 
+    `Case`s can be accessed by case number, eg `sim[0]`, and `InVar`s or
+    `OutVar`s can be accessed by name, eg `sim['Var1']`.
+
     Parameters
     ----------
     name : str
@@ -79,7 +92,7 @@ class Sim:
     invarseeds : list[int]
         The random seeds for each of the input variables.
     outvarseeds : list[int]
-        The random seeds for each of the input variables.
+        The random seeds for each of the output variables.
     caseseeds : list[int]
         The random seeds for each of the cases.
     inittime : datetime.datetime
@@ -91,11 +104,11 @@ class Sim:
     runtime : datetime.timedelta
         The length of time it took the simulation to run.
     casespreprocessed : set[int]
-        The case numbers which were sucessfully preprocessed.
+        The case numbers which were successfully preprocessed.
     casesrun : set[int]
-        The case numbers which were sucessfully run.
+        The case numbers which were successfully run.
     casespostprocessed : set[int]
-        The case numbers which were sucessfully postprocessed.
+        The case numbers which were successfully postprocessed.
     vars : dict[str, monaco.mc_var.InVar]
         All Variables.
     invars : dict[str, monaco.mc_var.InVar]
@@ -136,7 +149,7 @@ class Sim:
                  keepsimrawoutput  : bool = True,
                  savesimdata       : bool = False,
                  savecasedata      : bool = False,
-                 resultsdir        : str | pathlib.Path = None,
+                 resultsdir        : str | pathlib.Path | None = None,
                  ) -> None:
 
         self.checkFcnsInput(fcns)
@@ -148,7 +161,7 @@ class Sim:
         self.fcns = fcns
         self.firstcaseismedian = firstcaseismedian
         self.samplemethod = samplemethod
-        self.seed = seed
+        self.seed = int(seed)
         self.singlethreaded = singlethreaded
         self.daskkwargs = daskkwargs
         self.keepsiminput = keepsiminput
@@ -171,9 +184,9 @@ class Sim:
         self.caseseeds   : list[int] = []
 
         self.inittime  : datetime = datetime.now()
-        self.starttime : datetime = None
-        self.endtime   : datetime = None
-        self.runtime   : timedelta = None
+        self.starttime : datetime | None = None
+        self.endtime   : datetime | None = None
+        self.runtime   : timedelta | None = None
 
         self.casespreprocessed  : set[int] = set()
         self.casesrun           : set[int] = set()
@@ -187,11 +200,11 @@ class Sim:
         self.ninvars  : int = 0
         self.noutvars : int = 0
 
-        self.corrcoeffs : np.ndarray = None
-        self.covs       : np.ndarray = None
-        self.covvarlist : list[str] = None
+        self.corrcoeffs : np.ndarray | None = None
+        self.covs       : np.ndarray | None = None
+        self.covvarlist : list[str] | None = None
 
-        self.runsimid : int = None
+        self.runsimid : int | None = None
 
         self.ncases : int = ndraws + 1
         self.setFirstCaseMedian(firstcaseismedian)
@@ -200,7 +213,11 @@ class Sim:
         self.client = None
         self.cluster = None
         if not self.singlethreaded:
-            self.initDaskClient()
+            if HAS_DASK:
+                self.initDaskClient()
+            else:
+                vwarn(self.verbose, "Dask is not installed, running singlethreaded")
+                self.singlethreaded = True
 
 
     def __del__(self) -> None:
@@ -228,6 +245,30 @@ class Sim:
             self.initDaskClient()
 
 
+    def __getitem__(self,
+                    ncase_or_varname : int | str,
+                    ) -> Case | InVar | OutVar:
+        """Get a case, InVar, or OutVar from the simulation.
+
+        Parameters
+        ----------
+        ncase_or_varname : int | str
+            The case number or variable name to get.
+
+        Returns
+        -------
+        var : Case | InVar | OutVar
+            The case, InVar, or OutVar requested.
+        """
+        if isinstance(ncase_or_varname, int):
+            return self.cases[ncase_or_varname]
+        elif isinstance(ncase_or_varname, str):
+            return self.vars[ncase_or_varname]
+        else:
+            raise ValueError(f'Sim argument {ncase_or_varname=} must be an int ' +
+                             '(case number) or str (variable name)')
+
+
     def checkFcnsInput(self,
                        fcns: dict[SimFunctions, Callable],
                        ) -> None:
@@ -275,6 +316,10 @@ class Sim:
         """
         Initialize the dask distributed client.
         """
+        if not HAS_DASK:
+            vwarn(self.verbose, "Dask is not installed, skipping dask client initialization")
+            return
+
         if not self.singlethreaded:
             self.client = Client(**self.daskkwargs)
             self.cluster = self.client.cluster
@@ -290,27 +335,42 @@ class Sim:
 
     def addInVar(self,
                  name       : str,
-                 dist       : rv_discrete | rv_continuous,
-                 distkwargs : dict[str, Any],
+                 dist       : rv_discrete | rv_continuous | None = None,
+                 distkwargs : dict[str, Any]   = None,
                  nummap     : dict[float, Any] = None,
-                 seed       : int = None,
-                 datasource : Optional[str] = None,
+                 vals       : list[Any] | None = None,
+                 seed       : int | None       = None,
+                 datasource : Optional[str]    = None,
                  ) -> None:
         """
         Add an input variable to the simulation.
 
+        If `dist` is provided, then the variable will be drawn from a
+        statistical distribution. If `vals` is provided, then the variable will
+        be set to the provided values. Must provide one or the other. If
+        nonnumeric values are provided, then `nummap` must also be provided.
+
+        Examples
+        --------
+        >>> sim.addInVar(name='Var1', dist=scipy.stats.norm, distkwargs={'loc': 0, 'scale': 1})
+        >>> sim.addInVar(name='Var2', vals=[1, 2, 3])
+        >>> sim.addInVar(name='Var3', vals=['a', 'b', 'c'], nummap={0: 'a', 1: 'b', 2: 'c'})
+
         Parameters
         ----------
         name : str
             The name of this variable.
-        dist : scipy.stats.rv_discrete | scipy.stats.rv_continuous
-            The statistical distribution to draw from.
-        distkwargs : dict
+        dist : scipy.stats.rv_discrete | scipy.stats.rv_continuous | None
+            The statistical distribution to draw from. If None, must provide vals.
+        distkwargs : dict[str, Any] | None
             The keyword argument pairs for the statistical distribution
             function.
         nummap : dict[float, Any], default: None
             A dictionary mapping numbers to nonnumeric values.
-        seed : int
+        vals : list[Any] | None, default: None
+            Custom values to use instead of drawing from a distribution.
+            Length must match ncases.
+        seed : int | None
             The random seed for this variable. If None, a seed will be assigned
             based on the order added.
         datasource : str, default: None
@@ -325,9 +385,11 @@ class Sim:
             # seed is dependent on the order added
             seed = (self.seed + self.ninvars) % 2**32
         self.invarseeds.append(seed)
+
         invar = InVar(name=name, dist=dist, distkwargs=distkwargs, ndraws=self.ndraws,
-                      nummap=nummap, samplemethod=self.samplemethod, ninvar=self.ninvars,
-                      seed=seed, firstcaseismedian=self.firstcaseismedian, autodraw=False,
+                      nummap=nummap, vals=vals, samplemethod=self.samplemethod,
+                      ninvar=self.ninvars, seed=seed,
+                      firstcaseismedian=self.firstcaseismedian, autodraw=False,
                       datasource=datasource)
         self.invars[name] = invar
         self.vars[name] = invar
@@ -607,15 +669,13 @@ class Sim:
                             self.debug, self.verbose)
                         postprocessedcases[case.ncase] = casepostprocessed_delayed
 
-
                 if self.verbose:
+                    vprint(self.verbose, f'Preprocessing {len(casestopreprocess_downselect)}, ' +
+                                         f'running {len(casestorun_downselect)}, and ' +
+                                         f'postprocessing {len(casestopostprocess_downselect)} ' +
+                                         'cases...', end=' ', flush=True)
                     x = dask.persist(*postprocessedcases.values())
-                    n_tasks = (len(casestopreprocess_downselect)
-                              + len(casestorun_downselect)
-                              + len(casestopostprocess_downselect))
-                    tqdm_dask(x, total=n_tasks,
-                              desc='Preprocessing, running, and postprocessing cases',
-                              unit=' cases', position=0)
+                    progress(x)
                     fullyexecutedcases = dask.compute(*x)
                 else:
                     fullyexecutedcases = dask.compute(*postprocessedcases.values())
@@ -659,7 +719,7 @@ class Sim:
                 if case.ncase in cases_downselect:
                     case.haspreprocessed = False
                     case = preprocess_case(self.fcns[SimFunctions.PREPROCESS],
-                                            case, self.debug, self.verbose)
+                                           case, self.debug, self.verbose)
                     preprocessedcases.append(case)
                     if self.verbose:
                         pbar.update(1)
@@ -679,10 +739,10 @@ class Sim:
                         preprocessedcases.append(case_delayed)
 
                 if self.verbose:
+                    vprint(self.verbose, 'Preprocessing ' +
+                           f'{len(cases_downselect)} cases...', flush=True)
                     x = dask.persist(preprocessedcases)
-                    tqdm_dask(x, total=len(cases_downselect),
-                              desc='Preprocessing cases',
-                              unit=' cases', position=0)
+                    progress(x)
                     preprocessedcases = dask.compute(*x)[0]
                 else:
                     preprocessedcases = dask.compute(*preprocessedcases)
@@ -747,10 +807,10 @@ class Sim:
                         runcases.append(case_delayed)
 
                 if self.verbose:
+                    vprint(self.verbose, 'Running ' +
+                           f'{len(cases_downselect)} cases...', flush=True)
                     x = dask.persist(runcases)
-                    tqdm_dask(x, total=len(cases_downselect),
-                              desc='Running cases',
-                              unit=' cases', position=0)
+                    progress(x)
                     runcases = dask.compute(*x)[0]
                 else:
                     runcases = dask.compute(*runcases)
@@ -810,9 +870,9 @@ class Sim:
 
                 if self.verbose:
                     x = dask.persist(postprocessedcases)
-                    tqdm_dask(x, total=len(cases_downselect),
-                              desc='Postprocessing cases',
-                              unit=' cases', position=0)
+                    vprint(self.verbose, 'Postprocessing ' +
+                           f'{len(cases_downselect)} cases...', flush=True)
+                    progress(x)
                     postprocessedcases = dask.compute(*x)[0]
                 else:
                     postprocessedcases = dask.compute(*postprocessedcases)
@@ -873,7 +933,7 @@ class Sim:
             self.outvars[varname] = outvar
             self.vars[varname] = outvar
             for i in range(self.ncases):
-                self.cases[i].outvars[varname] = outvar
+                self.cases[i].addOutVar(outvar)
 
         self.noutvars = len(self.outvars)
 
@@ -891,8 +951,75 @@ class Sim:
         return scalaroutvars
 
 
+    def extendOutVars(self,
+                      outvars : None | str | Iterable[str] = None,
+                      ) -> None:
+        """
+        Extend the non-scalar output variables with their last value, so that
+        they all have the same shape. Can be useful for plotting or calculating
+        variable statistics.
+
+        Parameters
+        ----------
+        outvars : dict[str, OutVar], default: None
+            The output variables to extend. If None, then extends all the
+            output variables.
+        """
+        outvars_to_extend = []
+        if outvars is None:
+            outvars_to_extend = [outvar for outvar in self.outvars.values()
+                                 if not outvar.isscalar]
+        else:
+            outvars = get_list(outvars)
+            for name in outvars:
+                if not self.outvars[name].isscalar:
+                    outvars_to_extend.append(self.outvars[name])
+                else:
+                    vwarn(self.verbose, f"Output variable '{name}' is scalar," +
+                                         'skipping extension.')
+
+        outvars_extended = []
+        for outvar in outvars_to_extend:
+            n_points = max(len(outvar.vals[i]) for i in range(self.ncases))
+            for i in range(self.ncases):
+                vals = outvar.vals[i]
+                deficit = n_points - len(vals)
+                if deficit < 0:
+                    continue
+
+                if isinstance(vals, list):
+                    outvar.vals[i] = vals + [vals[-1]] * deficit
+                elif isinstance(vals, np.ndarray):
+                    rep = np.repeat(vals[-1][np.newaxis, ...], deficit, axis=0)
+                    outvar.vals[i] = np.concatenate([vals, rep], axis=0)
+                elif HAS_PANDAS and isinstance(vals, pd.Series):
+                    rep = pd.Series([vals.iloc[-1]] * deficit, index=range(len(vals), n_points))
+                    outvar.vals[i] = pd.concat([vals, rep])
+                elif HAS_PANDAS and isinstance(vals, pd.Index):
+                    rep = pd.Index([vals.iloc[-1]] * deficit, dtype=vals.dtype)
+                    outvar.vals[i] = pd.Index(np.concatenate([vals.to_numpy(), rep.to_numpy()]),
+                                              dtype=vals.dtype)
+                else:
+                    vwarn(self.verbose, f"Outvar '{outvar.name}' has an unsupported type" +
+                                        f"for extension: {type(vals)}. Skipping extension.")
+                    break
+
+            outvar_extended = OutVar(name=outvar.name, vals=outvar.vals, valmap=outvar.valmap,
+                                     ndraws=outvar.ndraws, seed=outvar.seed,
+                                     firstcaseismedian=outvar.firstcaseismedian,
+                                     datasource=outvar.datasource)
+            outvars_extended.append(outvar_extended)
+
+        for outvar in outvars_extended:
+            self.outvars[outvar.name] = outvar
+            self.vars[outvar.name] = outvar
+            for i in range(self.ncases):
+                self.cases[i].addOutVar(outvar)
+
+
     def calcSensitivities(self,
                           outvarnames : None | str | Iterable[str] = None,
+                          cases       : None | int | Iterable[int] = None,
                           tol         : float = 1e-6,
                           verbose     : bool = False,
                           ) -> None:
@@ -904,6 +1031,9 @@ class Sim:
         outvarnames : None | str | Iterable[str] (default: None)
             The outvar names to calculate sensitivity indices for. If None,
             then calculates sensitivities for all scalar outvars.
+        cases : None | int | Iterable[int], default: None
+            The cases to calculate sensitivities for. If None, then all cases
+            are used.
         tol : float, default 1e-6
             The convergence tolerance for scipy's minimize function acting on the
             negative log likelihood function.
@@ -921,6 +1051,7 @@ class Sim:
             else:
                 vprint(self.verbose, f"Calculating sensitivity indices for '{outvarname}'...")
                 sensitivities, ratios = calc_sensitivities(self, outvarname,
+                                                           cases=cases,
                                                            tol=tol, verbose=verbose)
 
                 sensitivities_dict = dict()
@@ -986,7 +1117,7 @@ class Sim:
 
 
     def plot(self,
-             scalarvars  : Optional[list[InVar | OutVar]] = None,
+             scalarvars  : Optional[list[InVar | OutVar | str]] = None,
              cases           : None | int | Iterable[int] = None,
              highlight_cases : None | int | Iterable[int] = empty_list(),
              rug_plot    : bool   = False,
@@ -1002,7 +1133,7 @@ class Sim:
 
         Parameters
         ----------
-        scalarvars : list[monaco.mc_var.InVar | monaco.mc_var.OutVar]
+        scalarvars : list[monaco.mc_var.InVar | monaco.mc_var.OutVar | str]
             The variables to plot. If None, then grabs all the input variables
             and scalar output variables.
         cases : None | int | Iterable[int], default: None
@@ -1037,8 +1168,22 @@ class Sim:
             for scalaroutvar in self.scalarOutVars().values():
                 scalarvars.append(scalaroutvar)
 
-        fig, axs = multi_plot_grid_rect(varsx=list(self.invars.values()),
-                                        varsy=list(self.scalarOutVars().values()),
+        scalarvars = get_list(scalarvars)
+
+        for i, scalarvar in enumerate(scalarvars):
+            if isinstance(scalarvar, str):
+                scalarvars[i] = self.vars[scalarvar]
+
+        invars = []
+        scalaroutvars = []
+        for scalarvar in scalarvars:
+            if isinstance(scalarvar, InVar):
+                invars.append(scalarvar)
+            elif isinstance(scalarvar, OutVar) and scalarvar.isscalar:
+                scalaroutvars.append(scalarvar)
+
+        fig, axs = multi_plot_grid_rect(varsx=invars,
+                                        varsy=scalaroutvars,
                                         cases=cases, highlight_cases=highlight_cases,
                                         rug_plot=rug_plot,
                                         cov_plot=cov_plot, cov_p=cov_p,
@@ -1121,10 +1266,29 @@ class Sim:
         """
         Export the nums for the selected to file for use externally.
 
+        The format for csv will be:
+
+        .. code-block:: text
+
+            "vars[0].name", "vars[1].name", "vars[2].name", ...
+            vars[0].nums[0], vars[1].nums[0], vars[2].nums[0], ...
+            vars[0].nums[1], vars[1].nums[1], vars[2].nums[1], ...
+            ...
+
+        The format for json will be:
+
+        .. code-block:: text
+
+            {
+                "vars[0].name": [vars[0].nums],
+                "vars[1].name": [vars[1].nums],
+                ...
+            }
+
         Parameters
         ----------
         vars : dict[str, InVar | OutVar]
-            The vars to save to file.
+            The vars to save to file. Will export the nums, not the vals.
         filename : Optional[str | pathlib.Path]
             The file to save to. Must be a csv or json.
             If a str, then will save in the resultsdir.
@@ -1180,6 +1344,7 @@ class Sim:
                      ) -> None:
         """
         Export the drawn nums for all the invars to file for use externally.
+        See `monaco.Sim.exportVars` docstring for csv and json formatting.
 
         Parameters
         ----------
@@ -1203,6 +1368,7 @@ class Sim:
                       ) -> None:
         """
         Export the nums for all the outvars to file for use externally.
+        See `monaco.Sim.exportVars` docstring for csv and json formatting.
 
         Parameters
         ----------
@@ -1227,6 +1393,25 @@ class Sim:
         """
         Import values from an external file.
 
+        The format for csv must be:
+
+        .. code-block:: text
+
+            "vars[0].name", "vars[1].name", "vars[2].name", ...
+            vars[0].nums[0], vars[1].nums[0], vars[2].nums[0], ...
+            vars[0].nums[1], vars[1].nums[1], vars[2].nums[1], ...
+            ...
+
+        The format for json must be:
+
+        .. code-block:: text
+
+            {
+                "vars[0].name": [vars[0].nums],
+                "vars[1].name": [vars[1].nums],
+                ...
+            }
+
         Parameters
         ----------
         filepath : str | pathlib.Path
@@ -1273,32 +1458,77 @@ class Sim:
 
 
     def importInVars(self,
-                     filepath : str | pathlib.Path,
-                     nummap   : dict[Any, float] = None,
+                     filepath     : str | pathlib.Path,
+                     dists        : Optional[list[rv_discrete | rv_continuous]] = None,
+                     distskwargs  : Optional[list[dict[str, Any]]] = None,
+                     nummaps      : Optional[list[dict[Any, float]]] = None,
                      ) -> None:
         """
-        Import draws from an external file as InVals.
+        Import draws from an external file as InVals. For each of the keyword
+        arguments, they must be the same length as the number of invars.
+        See `monaco.Sim.importVars` docstring for csv and json formatting.
 
         Parameters
         ----------
         filepath : str | pathlib.Path
             The file to load from. Must be a csv or json.
-        nummap : dict[Any, float], default: None
-            A nummap dict mapping numbers to nonnumeric values.
+        dists : list[rv_discrete | rv_continuous], default: None
+            A list of the distribution that was used for the draws. Needed if
+            it is desired to plot the analytical distribution or run a DVARS
+            sensitivity analysis. Note that for discrete distributions, the
+            calcuated percentiles will not likely match the original draws.
+        distskwargs : list[dict[str, Any]], default: None
+            A list of the distribution kwargs that were used for the draws.
+            Must be matched with `dists` above.
+        nummaps : list[dict[Any, float]], default: None
+            A list of nummap dicts mapping numbers to nonnumeric values.
         """
         vprint(self.verbose, 'Importing InVals from file...', flush=True)
 
         data, filepath = self.importVars(filepath)
 
-        pcts = [None for _ in range(self.ncases)]
-        for valname, nums in data.items():
-            self.addInVar(name=valname, dist=None, distkwargs=dict(), nummap=nummap,
+        if dists is not None and distskwargs is not None:
+            dists = get_list(dists)
+            distskwargs = get_list(distskwargs)
+            if len(dists) != len(data):
+                raise ValueError(f'Length of dists ({len(dists)}) ',
+                                 f'must match the number of invars ({len(data)}).')
+            if len(distskwargs) != len(data):
+                raise ValueError(f'Length of distskwargs ({len(distskwargs)}) ',
+                                 f'must match the number of invars ({len(data)}).')
+        else:
+            dists = [None for _ in range(len(data))]
+            distskwargs = [None for _ in range(len(data))]
+
+        if nummaps is not None:
+            nummaps = get_list(nummaps)
+            if len(nummaps) != len(data):
+                raise ValueError(f'Length of nummaps ({len(nummaps)}) ',
+                                 f'must match the number of invars ({len(data)}).')
+        else:
+            nummaps = [None for _ in range(len(data))]
+
+        for i, (valname, nums) in enumerate(data.items()):
+
+            if dists[i] is None or distskwargs[i] is None:
+                pcts = [None for _ in range(self.ncases)]
+                if distskwargs[i] is None:
+                    distskwargs[i] = dict()
+            else:
+                dist = dists[i](**distskwargs[i])
+                pcts = np.array(dist.cdf(nums))
+
+            self.addInVar(name=valname, dist=dists[i], distkwargs=distskwargs[i], nummap=nummaps[i],
                           seed=None, datasource=str(filepath.resolve()))
             nums = [np.array(num) for num in nums]
             self.invars[valname].nums = nums
             self.invars[valname].pcts = pcts
             self.invars[valname].mapNums()
 
+            for case in self.cases:
+                case.invars[valname] = self.invars[valname]
+                case.invals[valname] = self.invars[valname].getVal(case.ncase)
+
 
         vprint(self.verbose, f"InVals loaded from '{filepath.name}' and converted to variables",
                flush=True)
@@ -1306,25 +1536,46 @@ class Sim:
 
     def importOutVars(self,
                       filepath : str | pathlib.Path,
-                      valmap   : dict[Any, float] = None,
+                      nummaps  : Optional[list[dict[float, Any]]] = None,
                       ) -> None:
         """
         Import results from an external file as OutVals, convert to OutVars.
+        See `monaco.Sim.importVars` docstring for csv and json formatting.
 
         Parameters
         ----------
         filepath : str | pathlib.Path
             The file to load from. Must be a csv or json.
-        valmap : dict[Any, float], default: None
-            A valmap dict mapping nonnumeric values to numbers.
+        nummaps : list[dict[float, Any]], default: None
+            A list of nummap dicts mapping numbers to nonnumeric values. Note
+            that this is reversed from providing valmaps to OutVals.
         """
         vprint(self.verbose, 'Importing OutVals from file...', flush=True)
 
         data, filepath = self.importVars(filepath)
 
-        for case in self.cases:
-            for valname, vals in data.items():
-                case.addOutVal(valname, vals[case.ncase], valmap=valmap)
+        if nummaps is not None:
+            nummaps = get_list(nummaps)
+            if len(nummaps) != len(data):
+                raise ValueError(f'Length of nummaps ({len(nummaps)}) ',
+                                 f'must match the number of outvars ({len(data)}).')
+            valmaps = []
+            for nummap in nummaps:
+                if nummap is not None:
+                    valmaps.append({val: num for num, val in nummap.items()})
+                else:
+                    valmaps.append(None)
+        else:
+            nummaps = [None for _ in range(len(data))]
+            valmaps = [None for _ in range(len(data))]
+
+        for i, (valname, nums) in enumerate(data.items()):
+            if nummaps[i] is None:
+                vals = nums
+            else:
+                vals = [nummaps[i][num] for num in nums]
+            for case in self.cases:
+                case.addOutVal(valname, vals[case.ncase], valmap=valmaps[i])
 
         self.genOutVars(datasource=str(filepath.resolve()))
 
