@@ -2,14 +2,17 @@
 from __future__ import annotations
 
 import numpy as np
-from monaco.helper_functions import is_num, hashable_val, flatten
 from typing import Any
 from scipy.stats import rv_discrete, rv_continuous
 from abc import ABC
+from monaco.helper_functions import is_num, hashable_val, flatten
+
 try:
     import pandas as pd
+    HAS_PANDAS = True
 except ImportError:
     pd = None
+    HAS_PANDAS = False
 
 
 ### Val Base Class ###
@@ -25,19 +28,24 @@ class Val(ABC):
         The number of the case for this value.
     ismedian : bool
         Whether this case represents the median case.
+    datasource : str | None
+        If the value was imported from a file, this is the filepath. If
+        generated through monaco, then None.
     """
     def __init__(self,
-                 name     : str,
-                 ncase    : int,
-                 ismedian : bool,
+                 name       : str,
+                 ncase      : int,
+                 ismedian   : bool,
+                 datasource : str | None = None,
                  ):
         self.name = name
         self.ncase = ncase
         self.ismedian = ismedian
+        self.datasource = datasource
 
         self.val      : Any
         self.valmap   : dict[Any, float]
-        self.num      : np.float_ | np.ndarray
+        self.num      : np.float64 | np.ndarray
         self.nummap   : dict[float, Any]
         self.isscalar : bool
         self.shape    : tuple[int, ...]
@@ -64,7 +72,10 @@ class InVal(Val):
     nummap : dict[float, Any], default: None
         A dictionary mapping numbers to nonnumeric values.
     ismedian : bool, default: False
-        Whether this case represents the median case,
+        Whether this case represents the median case.
+    datasource : str | None, default: None
+        If the value was imported from a file, this is the filepath. If
+        generated through monaco, then None.
 
     Attributes
     ----------
@@ -80,19 +91,21 @@ class InVal(Val):
         `nummap`).
     """
     def __init__(self,
-                 name     : str,
-                 ncase    : int,
-                 pct      : float,
-                 num      : float,
-                 dist     : rv_discrete | rv_continuous,
-                 nummap   : dict[float, Any] = None,
-                 ismedian : bool = False,
+                 name       : str,
+                 ncase      : int,
+                 pct        : float,
+                 num        : float,
+                 dist       : rv_discrete | rv_continuous,
+                 nummap     : dict[float, Any] | None = None,
+                 ismedian   : bool = False,
+                 datasource : str | None = None,
                  ):
 
-        super().__init__(name=name, ncase=ncase, ismedian=ismedian)
+        super().__init__(name=name, ncase=ncase, ismedian=ismedian,
+                         datasource=datasource)
         self.dist = dist
         self.pct = pct
-        self.num = np.float_(num)
+        self.num = np.float64(num)
         self.nummap = nummap
         self.isscalar = True
         self.shape = ()
@@ -144,6 +157,9 @@ class OutVal(Val):
         A dictionary mapping nonnumeric values to numbers.
     ismedian : bool, default: False
         Whether this case represents the median case,
+    datasource : str | None, default: None
+        If the value was imported from a file, this is the filepath. If
+        generated through monaco, then None.
 
     Attributes
     ----------
@@ -161,13 +177,15 @@ class OutVal(Val):
         `valmap`).
     """
     def __init__(self,
-                 name     : str,
-                 ncase    : int,
-                 val      : Any,
-                 valmap   : dict[Any, float] = None,
-                 ismedian : bool = False,
+                 name       : str,
+                 ncase      : int,
+                 val        : Any,
+                 valmap     : dict[Any, float] | None = None,
+                 ismedian   : bool = False,
+                 datasource : str | None = None,
                  ):
-        super().__init__(name=name, ncase=ncase, ismedian=ismedian)
+        super().__init__(name=name, ncase=ncase, ismedian=ismedian,
+                         datasource=datasource)
         self.val = val
         self.valmap = valmap
         self.convertPandas()
@@ -257,7 +275,7 @@ class OutVal(Val):
 
     def split(self) -> dict[str, 'OutVal']:  # Quotes in typing to avoid import error
         """
-        Split a multidimentional output value along its outermost dimension,
+        Split a output value along its outermost dimension,
         and generate individual OutVal objects for each index.
 
         Returns
@@ -268,6 +286,10 @@ class OutVal(Val):
         if len(self.shape) > 1:
             for i in range(self.shape[0]):
                 name = self.name + f' [{i}]'
-                vals[name] = OutVal(name=name, ncase=self.ncase, val=self.val[i],
+                if HAS_PANDAS and isinstance(self.val, (pd.Series, pd.Index)):
+                    val = self.val.iloc[i]
+                else:
+                    val = self.val[i]
+                vals[name] = OutVal(name=name, ncase=self.ncase, val=val,
                                     valmap=self.valmap, ismedian=self.ismedian)
         return vals
