@@ -3,19 +3,18 @@ from __future__ import annotations
 
 import numpy as np
 from scipy.stats import rv_continuous, rv_discrete, describe
-from scipy.stats.stats import DescribeResult
 from matplotlib.figure import Figure
 from matplotlib.axes import Axes
+from copy import copy
+from typing import Any, Callable, Iterable, Optional, NamedTuple
+from warnings import warn
+from abc import ABC, abstractmethod
 from monaco.mc_val import Val, InVal, OutVal
 from monaco.mc_varstat import VarStat
 from monaco.mc_enums import SampleMethod, Sensitivities, VarStatType, InVarSpace
 from monaco.mc_sampling import sampling
 from monaco.mc_plot import plot, plot_sensitivities
-from monaco.helper_functions import empty_list, hashable_val
-from copy import copy
-from typing import Any, Callable, Iterable, Optional
-from warnings import warn
-from abc import ABC, abstractmethod
+from monaco.helper_functions import empty_list, hashable_val, flatten, get_list, is_num
 
 
 ### Var Base Class ###
@@ -26,16 +25,52 @@ class Var(ABC):
     Parameters
     ----------
     name : str
-        The name of this value.
+        The name of this variable.
     ndraws : int
         The number of random draws.
     seed : int
         The random seed to use for bootstrapping.
     firstcaseismedian : bool
         Whether the first case represents the median case.
-    datasource : str
+    datasource : str | None
         If the vals were imported from a file, this is the filepath. If
         generated through monaco, then None.
+
+    Attributes
+    ----------
+    name : str
+        The name of this variable.
+    ndraws : int
+        The number of random draws.
+    seed : int
+        The random seed to use for bootstrapping.
+    firstcaseismedian : bool
+        Whether the first case represents the median case.
+    datasource : str | None
+        If the vals were imported from a file, this is the filepath. If
+        generated through monaco, then None.
+    ncases : int
+        The number of cases, which is `ndraws + 1` if `firstcaseismedian` and
+        `ndraws` otherwise.
+    vals : list[Any]
+        The values corresponding to the randomly drawn numbers. If valmap is
+        None, then `vals == nums.tolist()`.
+    valmap : dict[Any, float]
+        A dictionary mapping nonnumeric values to numbers (the inverse of
+        `nummap`).
+    nums : list[np.ndarray]
+        The randomly drawn numbers obtained by feeding `pcts` into `dist`.
+    nummap : dict[float, Any]
+        A dictionary mapping numbers to nonnumeric values (the inverse of
+        `valmap`).
+    pcts : list[float]
+        The randomly drawn percentiles.
+    maxdim : int
+        The maximum dimension of the values.
+    isscalar : bool
+        Whether this is a scalar variable.
+    varstats : list[moncao.mc_varstat.VarStat]
+        A list of all the variable statistics for this variable.
     """
     def __init__(self,
                  name              : str,
@@ -46,7 +81,7 @@ class Var(ABC):
                  ):
         self.name = name
         self.ndraws = ndraws
-        self.seed = seed
+        self.seed = int(seed)
         self.firstcaseismedian = firstcaseismedian
         self.datasource = datasource
 
@@ -62,6 +97,24 @@ class Var(ABC):
         self.varstats : list[VarStat] = empty_list()
 
 
+    def __getitem__(self,
+                    ncase : int,
+                    ) -> Val:
+        """Get a value from the variable.
+
+        Parameters
+        ----------
+        ncase : int
+            The number of the case to get the value for.
+
+        Returns
+        -------
+        val : Val
+            The value requested.
+        """
+        return self.getVal(ncase)
+
+
     def setFirstCaseMedian(self,
                            firstcaseismedian : bool,
                            ) -> None:
@@ -81,14 +134,14 @@ class Var(ABC):
             self.ncases = self.ndraws
 
 
-    def stats(self) -> DescribeResult:
+    def stats(self) -> NamedTuple:
         """
         Calculates statistics of the variable nums from scipy.stats.describe.
 
         Returns
         -------
-        stats : DescribeResult
-            A dict with descriptive statistics for the variable nums.
+        stats : NamedTuple
+            A named tuple with descriptive statistics for the variable nums.
         """
         stats = describe(self.nums)
         return stats
@@ -96,12 +149,13 @@ class Var(ABC):
 
     def addVarStat(self,
                    stat        : VarStatType | Callable,
-                   statkwargs  : dict[str, Any] = None,
+                   statkwargs  : dict[str, Any] | None = None,
+                   cases       : None | int | Iterable[int] = None,
                    bootstrap   : bool = False,
                    bootstrap_k : int = 10,
                    conf        : float = 0.95,
-                   seed        : int = None,
-                   name        : str = None,
+                   seed        : int | None = None,
+                   name        : str | None = None,
                    ) -> None:
         """
         Add a variable statistic to this variable.
@@ -111,11 +165,87 @@ class Var(ABC):
         stat : monaco.mc_enums.VarStatType | Callable
             The type of variable statistic to add.
         statkwargs : dict[str, Any]
-            Keyword arguments for the specified variable stastistic.
+            Keyword arguments for the specified variable statistic.
+        cases : None | int | Iterable[int]
+            The cases to use to calculate the statistic. If None, then all cases
+            are used.
+        bootstrap : bool (default: True)
+            Whether to use bootstrapping to generate confidence intervals for
+            the statistic.
+        bootstrap_k : int (default: 10)
+            The k'th order statistic to determine the number of bootstrap draws
+            for the given confidence level. Must be >= 1. Set higher for a
+            smoother bootstrap distribution.
+        conf : float (default: 0.95)
+            The confidence level for the confidence interval.
         seed : int
             The random seed to use for bootstrapping.
         name : str
             The name of the variable statistic to add.
+
+        Notes
+        -----
+        These are the valid stats with their statkwargs
+
+        max()
+            ``'max'``, no kwargs
+        min()
+            ``'min'``, no kwargs
+        median()
+            ``'median'``, no kwargs
+        mean()
+            ``'mean'``, no kwargs
+        geomean()
+            ``'geomean'``, no kwargs
+        mode()
+            ``'mode'``, no kwargs
+        variance()
+            ``'variance'``, no kwargs
+        skewness()
+            ``'skewness'``, no kwargs
+        kurtosis()
+            ``'kurtosis'``, no kwargs
+        moment(n : int)
+            ``'moment'``
+
+            `n` is the n'th moment, `n > 0`.
+        percentile(p : float)
+            ``'percentile'``
+
+            `p` is the percentile, `0 < p < 1`.
+        sigma(sig : float, bound : monaco.mc_enums.StatBound)
+            ``'sigma'``
+
+            `sig` is the gaussian sigma value, `-inf < sig < inf`.
+
+            `bound` is the statistical bound, ether `'1-sided'` or `'2-sided'`.
+            Default is `'2-sided'`.
+        gaussianP(p : float, bound : monaco.mc_enums.StatBound)
+            ``'gaussianP'``
+
+            `p` is the percentile, `0 < p < 1`.
+
+            `bound` is the statistical bound, ether `'1-sided'` or `'2-sided'`.
+            Default is `'2-sided'`.
+        orderstatTI(p : float, c : float, bound : monaco.mc_enums.StatBound)
+            ``'orderstatTI'``
+
+            `p` is the percentage, `0 < p < 1`
+
+            `c` is the confidence, `0 < c < 1`. Default is `0.95`.
+
+            `bound` is the statistical bound, ether `'1-sided'`, `'2-sided'`, or
+            `'all'`. Default is `'2-sided'`.
+        orderstatP(p : float, c : float, bound : monaco.mc_enums.StatBound)
+            ``'orderstatP'``
+
+            `p` is the percentage, `0 < p < 1`
+
+            `c` is the confidence, `0 < c < 1`. Default is `0.95`.
+
+            `bound` is the statistical bound, ether `'1-sided lower'`,
+            `'1-sided upper'`, `'2-sided'`, `'all'`, or '`nearest'`. Default is
+            `'2-sided'`.
         """
         if statkwargs is None:
             statkwargs = dict()
@@ -124,6 +254,7 @@ class Var(ABC):
             seed = (self.seed + 1 + len(self.varstats)) % 2**32
 
         self.varstats.append(VarStat(var=self, stat=stat, statkwargs=statkwargs,
+                                     cases=cases,
                                      bootstrap=bootstrap, bootstrap_k=bootstrap_k,
                                      conf=conf, seed=seed, name=name))
 
@@ -136,8 +267,8 @@ class Var(ABC):
 
 
     def plot(self,
-             vary   : InVar | OutVar = None,
-             varz   : InVar | OutVar = None,
+             vary   : InVar | OutVar | None = None,
+             varz   : InVar | OutVar | None = None,
              cases           : None | int | Iterable[int] = None,
              highlight_cases : None | int | Iterable[int] = empty_list(),
              rug_plot    : bool           = False,
@@ -172,6 +303,8 @@ class InVar(Var):
     """
     A Monte Carlo input variable.
 
+    `InVal`s can be accessed by case number, eg `invar[0]`.
+
     Parameters
     ----------
     name : str
@@ -195,7 +328,7 @@ class InVar(Var):
         Whether the first case represents the median case.
     autodraw : bool, default: True
         Whether to draw the random values when this variable is created.
-    datasource : str, default: None
+    datasource : str | None, default: None
         If the invals were imported from a file, this is the filepath. If
         generated through monaco, then None.
 
@@ -221,9 +354,10 @@ class InVar(Var):
     def __init__(self,
                  name              : str,
                  ndraws            : int,
-                 dist              : rv_discrete | rv_continuous,
-                 distkwargs        : dict             = None,
+                 dist              : rv_discrete | rv_continuous | None = None,
+                 distkwargs        : dict[str, Any] | None = None,
                  nummap            : dict[float, Any] = None,
+                 vals              : list[Any]        = None,
                  samplemethod      : SampleMethod     = SampleMethod.SOBOL_RANDOM,
                  ninvar            : int              = None,
                  seed              : int = np.random.get_state(legacy=False)['state']['key'][0],
@@ -235,7 +369,20 @@ class InVar(Var):
                          firstcaseismedian=firstcaseismedian,
                          datasource=datasource)
 
+        # Validate inputs for custom values case
+        if dist is None and vals is None:
+            raise ValueError("Either 'dist' or 'vals' must be provided")
+        if dist is not None and vals is not None:
+            raise ValueError("Cannot provide both 'dist' and 'vals', choose one")
+        if vals is not None:
+            samplemethod = None
+            vals = get_list(vals)
+            if len(vals) != self.ncases:
+                raise ValueError(f"Length of 'vals' ({len(vals)}) must match " +
+                                 f"ncases ({self.ncases})")
+
         self.dist = dist
+        self.custom_vals = vals
         if distkwargs is None:
             distkwargs = dict()
         self.distkwargs = distkwargs
@@ -294,6 +441,7 @@ class InVar(Var):
         """
         Perform the random draws based on the sampling method and the
         statistical distribution, and map those draws to values.
+        Alternatively, use custom values if provided.
 
         Parameters
         ----------
@@ -302,13 +450,35 @@ class InVar(Var):
         """
         self.pcts = []
         self.nums = []
-        dist = self.dist(**self.distkwargs)
 
+        # Custom values
+        if self.custom_vals is not None:
+            self.vals = copy(self.custom_vals)
+
+            if self.valmap is not None:
+                for val in self.vals:
+                    self.nums.append(np.array(self.valmap[val]))
+            else:
+                if any(not is_num(val) for val in self.vals):
+                    raise ValueError("Must supply 'nummap' if nonnumeric custom " +
+                                     "values are provided")
+                self.nums = [np.array(val) for val in self.vals]
+
+            # Generate uniform percentiles for custom values
+            self.pcts = [i / (self.ndraws - 1) for i in range(self.ndraws)]
+            if self.firstcaseismedian:
+                # First gets median, and we deconflict 0.5 with a small offset
+                self.pcts = [0.5] + [p if p != 0.5 else p + 1e-12 for p in self.pcts]
+
+            return
+
+        # Distribution-based drawing logic
         if self.firstcaseismedian:
             self.ncases = self.ndraws + 1
             self.pcts.append(0.5)
             self.nums.append(np.array(self.getDistMedian()))
 
+        dist = self.dist(**self.distkwargs)
         pcts = sampling(ndraws=self.ndraws, method=self.samplemethod,
                         ninvar=self.ninvar, ninvar_max=ninvar_max,
                         seed=self.seed)
@@ -353,7 +523,7 @@ class InVar(Var):
         val = InVal(name=self.name, ncase=ncase,
                     pct=self.pcts[ncase], num=self.nums[ncase].item(),
                     dist=self.dist, nummap=self.nummap,
-                    ismedian=ismedian)
+                    ismedian=ismedian, datasource=self.datasource)
         return val
 
 
@@ -407,6 +577,8 @@ class OutVar(Var):
     """
     A Monte Carlo output variable.
 
+    `OutVal`s can be accessed by case number, eg `outvar[0]`.
+
     Parameters
     ----------
     name : str
@@ -421,7 +593,7 @@ class OutVar(Var):
         The random seed for bootstrapping.
     firstcaseismedian : bool, default: False
         Whether the first case represents the median case.
-    datasource : str, default: None
+    datasource : str | None, default: None
         If the outvals were imported from a file, this is the filepath. If
         generated through monaco, then None.
 
@@ -437,7 +609,7 @@ class OutVar(Var):
     nums : list[np.ndarry]
         The numbers corresponding to the output values. If valmap is None, then
         `nums == list[np.array(vals)]`.
-    varstats : list[moncao.VarStat.VarStat]
+    varstats : list[monaco.VarStat.VarStat]
         A list of all the variable statistics for this variable.
     """
     def __init__(self,
@@ -486,17 +658,26 @@ class OutVar(Var):
         """
         Parse the output values and extract a valmap.
         """
-        Val0 = self.getVal(0)
-        if Val0.valmap is not None:
-            if Val0.valmapsource == 'auto':
-                uniquevals : set[Any] = set()
-                for i in range(self.ncases):
+        if self.getVal(0).valmapsource == 'assigned':
+            self.valmap = self.getVal(0).valmap
+            return
+
+        if any([self.getVal(i).valmap is not None for i in range(self.ncases)]):
+            uniquevals : set[Any] = set()
+            for i in range(self.ncases):
+                if self.getVal(i).valmap is not None:
                     uniquevals.update(self.getVal(i).valmap.keys())
-                self.valmap = dict()
-                for i, val in enumerate(sorted(uniquevals)):
-                    self.valmap[val] = i
-            else:
-                self.valmap = Val0.valmap
+                else:
+                    uniquevals.update(set(flatten([self.getVal(i).val])))
+
+            # Sort values, handling None values by putting them first
+            sorted_vals = sorted([x for x in uniquevals if x is not None])
+            if None in uniquevals:
+                sorted_vals.insert(0, None)
+
+            self.valmap = dict()
+            for i, val in enumerate(sorted_vals):
+                self.valmap[val] = i
 
 
     def genNumMap(self) -> None:
@@ -537,7 +718,8 @@ class OutVar(Var):
             ismedian = True
 
         val = OutVal(name=self.name, ncase=ncase, val=self.vals[ncase],
-                     valmap=self.valmap, ismedian=ismedian)
+                     valmap=self.valmap, ismedian=ismedian,
+                     datasource=self.datasource)
         return val
 
 
@@ -559,7 +741,7 @@ class OutVar(Var):
 
     def split(self) -> dict[str, 'OutVar']:  # Quotes in typing to avoid import error
         """
-        Split a multidimentional output variable along its outermost dimension,
+        Split a multidimensional output variable along its outermost dimension,
         and generate individual OutVar objects for each index.
 
         Returns
