@@ -11,12 +11,13 @@ from copy import copy
 from statistics import mode
 from scipy.stats import bootstrap, moment, skew, kurtosis
 from scipy.stats.mstats import gmean
-from monaco.helper_functions import get_list
+from typing import Any, Callable, Iterable
+from warnings import warn
+from monaco.helper_functions import get_list, get_cases
 from monaco.gaussian_statistics import pct2sig, sig2pct
 from monaco.order_statistics import (order_stat_TI_n, order_stat_TI_k,
                                      order_stat_P_k, get_iP)
 from monaco.mc_enums import StatBound, VarStatType, VarStatSide
-from typing import Any, Callable
 
 
 class VarStat:
@@ -32,6 +33,9 @@ class VarStat:
         accept an "axis" kwarg for bootstrap vectorization.
     statkwargs : dict[str:Any]
         The keyword arguments for the variable statistic.
+    cases : None | int | Iterable[int], default: None
+        The cases to use when calculating the statistic. If None, then all cases
+        are used.
     bootstrap : bool (default: True)
         Whether to use bootstrapping to generate confidence intervals for the
         statistic.
@@ -56,6 +60,10 @@ class VarStat:
         The nums for the high side of the confidence interval (None if no CI).
     vals : list[Any]
         The values for the `nums` as determined by `var.nummap`
+    cases : list[int]
+        The cases used to calculate the statistic.
+    ncases : int
+        The number of cases used to calculate the statistic.
     confidence_interval_low_vals : list[Any]
         The values for `confidence_interval_low_nums` via `var.nummap`
     confidence_interval_high_vals : list[Any]
@@ -68,38 +76,48 @@ class VarStat:
     These are the valid stats with their statkwargs
 
     max()
-        No kwargs
+        ``'max'``, no kwargs
     min()
-        No kwargs
+        ``'min'``, no kwargs
     median()
-        No kwargs
+        ``'median'``, no kwargs
     mean()
-        No kwargs
+        ``'mean'``, no kwargs
     geomean()
-        No kwargs
+        ``'geomean'``, no kwargs
     mode()
-        No kwargs
+        ``'mode'``, no kwargs
     variance()
-        No kwargs
+        ``'variance'``, no kwargs
     skewness()
-        No kwargs
+        ``'skewness'``, no kwargs
     kurtosis()
-        No kwargs
+        ``'kurtosis'``, no kwargs
     moment(n : int)
+        ``'moment'``
+
         `n` is the n'th moment, `n > 0`.
     percentile(p : float)
+        ``'percentile'``
+
         `p` is the percentile, `0 < p < 1`.
     sigma(sig : float, bound : monaco.mc_enums.StatBound)
+        ``'sigma'``
+
         `sig` is the gaussian sigma value, `-inf < sig < inf`.
 
         `bound` is the statistical bound, ether `'1-sided'` or `'2-sided'`.
         Default is `'2-sided'`.
     gaussianP(p : float, bound : monaco.mc_enums.StatBound)
+        ``'gaussianP'``
+
         `p` is the percentile, `0 < p < 1`.
 
         `bound` is the statistical bound, ether `'1-sided'` or `'2-sided'`.
         Default is `'2-sided'`.
     orderstatTI(p : float, c : float, bound : monaco.mc_enums.StatBound)
+        ``'orderstatTI'``
+
         `p` is the percentage, `0 < p < 1`
 
         `c` is the confidence, `0 < c < 1`. Default is `0.95`.
@@ -107,6 +125,8 @@ class VarStat:
         `bound` is the statistical bound, ether `'1-sided'`, `'2-sided'`, or
         `'all'`. Default is `'2-sided'`.
     orderstatP(p : float, c : float, bound : monaco.mc_enums.StatBound)
+        ``'orderstatP'``
+
         `p` is the percentage, `0 < p < 1`
 
         `c` is the confidence, `0 < c < 1`. Default is `0.95`.
@@ -117,21 +137,27 @@ class VarStat:
     """
     def __init__(self,
                  var         : Var,
-                 stat        : VarStatType | Callable,
-                 statkwargs  : dict[str, Any] = None,
+                 stat        : str | VarStatType | Callable,
+                 statkwargs  : dict[str, Any] | None = None,
+                 cases       : None | int | Iterable[int] = None,
                  bootstrap   : bool = True,
                  bootstrap_k : int = 10,
                  conf        : float = 0.95,
                  seed        : int = np.random.get_state(legacy=False)['state']['key'][0],
-                 name        : str = None,
+                 name        : str | None = None,
                  ):
 
         self.var = var
+        if isinstance(stat, str):
+            stat = stat.lower()
         self.stat = stat
         if statkwargs is None:
             statkwargs = dict()
         self.statkwargs = statkwargs
 
+        self.cases = get_cases(ncases=self.var.ncases, cases=cases)
+        self.ncases = len(self.cases)
+
         self.nums : np.ndarray = np.array([])
         self.vals : list[Any] | np.ndarray = []
         self.name = name
@@ -141,11 +167,11 @@ class VarStat:
             raise ValueError(f'bootstrap_k = {bootstrap_k} must be >= 1')
         self.bootstrap_k = bootstrap_k
         self.conf = conf
-        self.confidence_interval_low_nums : np.ndarray = None
-        self.confidence_interval_high_nums : np.ndarray = None
+        self.confidence_interval_low_nums : list | np.ndarray = None
+        self.confidence_interval_high_nums : list | np.ndarray = None
         self.confidence_interval_low_vals : list[Any] | np.ndarray = []
         self.confidence_interval_high_vals : list[Any] | np.ndarray = []
-        self.bootstrap_n = None
+        self.bootstrap_n : int | None = None
         self.seed = seed
 
         if isinstance(stat, Callable):
@@ -198,6 +224,9 @@ class VarStat:
                              f'{VarStatType.GAUSSIANP}, {VarStatType.ORDERSTATTI}, ' +
                              f'{VarStatType.ORDERSTATP}')
 
+        if self.ncases != self.var.ncases:
+            warn(f'Only using {self.ncases} of {self.var.ncases} cases for VarStat {self.name}')
+
 
     def genStatsMoment(self) -> None:
         """
@@ -262,7 +291,7 @@ class VarStat:
     def sigma(self,
               x,  # TODO: explicit typing here
               sig  : float,
-              axis : int = None,
+              axis : int | None = None,
               ) -> float:
         """
         Calculate the sigma value of a normally distributed list of numbers.
@@ -282,7 +311,7 @@ class VarStat:
 
     def statsFunctionWrapper(self,
                              x : Any,
-                             axis : int = None,  # Needed for bootstrap vectorization
+                             axis : int | None = None,  # Needed for bootstrap vectorization
                              ) -> Any:
         """
         A wrapper function to allow using a bootstrap function that uses kwargs.
@@ -323,13 +352,13 @@ class VarStat:
         # Scalar Variables
         if self.var.isscalar:
             # Calculate nums and confidence interval for each point in the sequence
-            self.nums = self.statsFunctionWrapper(self.var.nums)
+            nums = [self.var.nums[case] for case in self.cases]
+            self.nums = self.statsFunctionWrapper(nums)
             if self.bootstrap:
-                # Switch to method='Bca' once https://github.com/scipy/scipy/issues/15883 resolved
-                res = bootstrap((np.array(self.var.nums),), self.statsFunctionWrapper,
+                res = bootstrap((np.array(nums),), self.statsFunctionWrapper,
                                 confidence_level=self.conf,
                                 n_resamples=self.bootstrap_n,
-                                random_state=self.seed, method='basic')
+                                random_state=self.seed, method='BCa')
                 self.confidence_interval_low_nums = res.confidence_interval.low
                 self.confidence_interval_high_nums = res.confidence_interval.high
 
@@ -348,7 +377,7 @@ class VarStat:
 
         # 1-D Variables
         elif self.var.maxdim == 1:
-            nums_list = get_list(self.var.nums)
+            nums_list = [self.var.nums[case] for case in self.cases]
             npoints = max(len(x) for x in nums_list)
             if self.bootstrap:
                 confidence_interval_low_nums = []
@@ -418,10 +447,10 @@ class VarStat:
                      f'{bound_str} P{round(self.p*100,4)}/{round(self.c*100,4)}% ' +
                       'Confidence Interval')
 
-        self.k = order_stat_TI_k(n=self.var.ncases, p=self.p, c=self.c, bound=self.bound)
+        self.k = order_stat_TI_k(n=self.ncases, p=self.p, c=self.c, bound=self.bound)
 
         if self.var.isscalar:
-            sortednums = sorted(self.var.nums)
+            sortednums = sorted([self.var.nums[case] for case in self.cases])
             if self.side == VarStatSide.LOW:
                 sortednums.reverse()
             if self.side in (VarStatSide.HIGH, VarStatSide.LOW):
@@ -445,14 +474,15 @@ class VarStat:
                 self.vals = copy(self.nums)
 
         elif self.var.maxdim == 1:
-            npoints = max(x.shape[0] if len(x.shape) > 0 else 0 for x in self.var.nums)
+            nums = [self.var.nums[case] for case in self.cases]
+            npoints = max(x.shape[0] if len(x.shape) > 0 else 0 for x in nums)
             self.nums = np.empty(npoints)
             if self.side == VarStatSide.BOTH:
                 self.nums = np.empty((npoints, 2))
             elif self.side == VarStatSide.ALL:
                 self.nums = np.empty((npoints, 3))
             for i in range(npoints):
-                numsatidx = [x[i] for x in self.var.nums
+                numsatidx = [x[i] for x in nums
                              if (len(x.shape) > 0 and x.shape[0] > i)]
                 sortednums = sorted(numsatidx)
                 if self.side == VarStatSide.LOW:
@@ -491,11 +521,11 @@ class VarStat:
                      f'{self.bound} {self.c*100}% Confidence Bound around ' +
                      f'{self.p*100}th Percentile')
 
-        self.k = order_stat_P_k(n=self.var.ncases, P=self.p, c=self.c, bound=bound)
+        self.k = order_stat_P_k(n=self.ncases, P=self.p, c=self.c, bound=bound)
 
-        (iPl, iP, iPu) = get_iP(n=self.var.ncases, P=self.p)
+        (iPl, iP, iPu) = get_iP(n=self.ncases, P=self.p)
         if self.var.isscalar:
-            sortednums = sorted(self.var.nums)
+            sortednums = sorted([self.var.nums[case] for case in self.cases])
             if self.bound == StatBound.ONESIDED_LOWER:
                 self.nums = np.array(sortednums[iPl - self.k])
             elif self.bound == StatBound.ONESIDED_UPPER:
@@ -524,14 +554,15 @@ class VarStat:
                 self.vals = copy(self.nums)
 
         elif self.var.maxdim == 1:
-            npoints = max(len(get_list(x)) for x in self.var.nums)
+            nums = [self.var.nums[case] for case in self.cases]
+            npoints = max(len(get_list(x)) for x in nums)
             self.nums = np.empty(npoints)
             if self.bound == StatBound.TWOSIDED:
                 self.nums = np.empty((npoints, 2))
             elif self.bound == StatBound.ALL:
                 self.nums = np.empty((npoints, 3))
             for i in range(npoints):
-                numsatidx = [get_list(x)[i] for x in self.var.nums if len(get_list(x)) > i]
+                numsatidx = [get_list(x)[i] for x in nums if len(get_list(x)) > i]
                 sortednums = sorted(numsatidx)
                 if self.bound == StatBound.ONESIDED_LOWER:
                     self.nums[i] = sortednums[iPl - self.k]
